package com.jpattern.service.cache.simple;

import java.io.Serializable;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import com.jpattern.service.cache.ACache;
import com.jpattern.service.cache.statistics.ICacheStatistics;

/**
 * 
 * A Cache that holds objects for a fixed maximum time
 * 
 * @author Francesco Cina
 *
 * 26/lug/2011
 */
public class FixedSizeTimedCache extends ACache {

	private static final long serialVersionUID = 1L;

	private final Map<String, ICacheElement> map;
	private final long timeToLiveMilliSeconds;
	public final int maxSize; 
	
	public FixedSizeTimedCache(String name, int maxSize, long timeToLiveSeconds, ICacheStatistics cacheStatistics) {
		super(name, cacheStatistics);
		this.maxSize = maxSize;
		this.timeToLiveMilliSeconds = timeToLiveSeconds*1000;
		
		map = new LinkedHashMap<String, ICacheElement>() {
			private static final long serialVersionUID = 1L;
			
            protected boolean removeEldestEntry(Map.Entry<String, ICacheElement> oldest) {
                return size() > FixedSizeTimedCache.this.maxSize;
            }
			
		};
	}
	
	@Override
	public synchronized Serializable getValue(String key) {
		ICacheElement cacheElement = map.get(key);
		if ( cacheElement!=null ) {
			if ( cacheElement.getTimestamp() < new Date().getTime()) {
				remove(key);
				return null;
			}
			return cacheElement.getValue();
		}
		return null;
	}

	@Override
	public synchronized void put(String key, Serializable value) {
		map.put(key, new CacheElement(value, new Date().getTime() + timeToLiveMilliSeconds));
	}

	@Override
	public synchronized void remove(String key) {
		map.remove(key);
	}

	@Override
	public synchronized void clear() {
		map.clear();
	}

	@Override
	public boolean contains(String key) {
		return map.containsKey(key);
	}
	
	@Override
	public int getSize() {
		return map.size();
	}
	
}
